varying mediump vec4 var_position;
varying mediump vec2 var_texcoord0;

uniform lowp sampler2D tex_light; // lighting render target texture
uniform lowp sampler2D tex_base; // base render target texture

void main()
{
	// This is basically a crude attempt at HDR tone mapping

	// The shader on the lights compresses their rgb values down to 1/8th...
	//    ...so multiple lights can be combined in the render target texture without losing information.
	// This shader messes around with the combined light values, then multiplies them back up by 8x.

	vec4 light = texture2D(tex_light, var_texcoord0.xy);
	vec4 base = texture2D(tex_base, var_texcoord0.xy);


	float thresh_min = 0.2;
	float thresh_max = 0.8;
	float thresh_range = thresh_max - thresh_min;

	// For each channel, if it's greater than a certain threshold, raise the other channels (the overall color approaches white)
	if (light.x > thresh_min) { 							// RED
		float a = (light.x - thresh_min)/thresh_range;
		a *= a;
		light.y = max(light.y, a * 0.3); // green
		light.z = max(light.z, a * 0.3); // blue
	}
	if (light.y > thresh_min) { 							// GREEN
		float a = (light.y - thresh_min)/thresh_range;
		a *= a;
		light.x = max(light.x, a * 0.45); // red
		light.z = max(light.z, a * 0.35); // blue
	}
	if (light.z > thresh_min) { 							// BLUE
		float a = (light.z - thresh_min)/thresh_range;
		a *= a;
		light.x = max(light.x, a * 0.35); // red
		light.y = max(light.y, a * 0.55); // green
	}

	light *= 4.0;
	light.xyz = pow(light.xyz, vec3(0.75)); // this exponent gives a nicer curve, but also some visible banding because of precision I guess
	base *= 2.0 * light;

	gl_FragColor = base;
}

